
var storage = function() {
	var debug = true;
	var session = {usid:"", username:""};
	session.usid = localStorage.getItem('usid');
	session.username = localStorage.getItem('username');

	/* Login attemts to login using the provided username and password. The
	 * provided callback is called with an object with a field named usid,
	 * which can be used to store objects in the database later. If an error 
	 * occurs, or an uexpected value is received, a object with the field
	 * error describing the error is returned.
	 */
	var login = function(user, pass, callback) {
		var client = new XMLHttpRequest();
		client.onreadystatechange = function() {
			if (this.readyState != this.DONE) return;
			var data = JSON.parse(this.responseText);
			var ret = {};
			if (data === undefined) {
				ret.error = "Invalid response from backend";
			} else if (typeof(data["error"]) === "string" && data["error"] !== "") {
				ret.error = data["error"];
			} else if (typeof(data["usid"]) !== "string" || data["usid"] === "") {
				ret.error = "Invalid response from backend";
			} else {
				ret.usid = data["usid"];
				session.usid = ret.usid;
				session.username = user;
				localStorage.setItem("username",user);
				localStorage.setItem("usid",ret.usid);
			}
			callback(ret);
		}
		var fd = new FormData();
		fd.append("username",user);
		fd.append("password",pass);
		client.open("POST", "/resource/login");
		client.send(fd);
		if (debug) console.log("login: request sent");
	}

	var register = function(user, pass, email, callback) {
		var client = new XMLHttpRequest();
		client.onreadystatechange = function() {
			if (this.readyState != this.DONE) return;
			if (debug) console.log(this.responseText);
			var data = JSON.parse(this.responseText);
			var ret = {};
			if (data === undefined) {
				ret.error = "Invalid response from backend";
			} else if (typeof(data["error"]) === "string" && data["error"] !== "") {
				ret.error = data["error"];
			} else if (typeof(data["usid"]) !== "string" || data["usid"] === "") {
				ret.error = "Invalid response from backend";
			} else {
				ret.usid = data["usid"];
				session.usid = ret.usid;
				session.username = user;
				localStorage.setItem("username",user);
				localStorage.setItem("usid",ret.usid);
			}
			callback(ret);
		}
		var fd = new FormData();
		fd.append("username",user);
		fd.append("password",pass);
		fd.append("email",email);
		client.open("POST", "/resource/register");
		client.send(fd);
		if (debug) console.log("login: request sent");
	}

	var logout = function(callback) {
		var client = new XMLHttpRequest();
		client.onreadystatechange = function() {
			if (this.readyState != this.DONE) return;
			console.log("logout completed");
			callback();
		}
		var fd = new FormData();
		fd.append("username",session.username);
		fd.append("usid",session.usid);
		client.open("POST", "/resource/logout");
		client.send(fd);
		session.usid = null;
		session.username = null;
	}

	var load = function(obj, rev, callback) {
		var client = new XMLHttpRequest();
		client.onreadystatechange = function() {
			if (debug) console.log("storage: readyState: " + this.readyState);
			if (this.readyState == this.DONE) {
				if (debug) console.log("storage: aquired object");
				callback(this.responseText);
			}
		};
		var fd = new FormData();
		fd.append("obj", obj);
		if (rev !== undefined) fd.append("rev", rev);
		client.open("POST", "/resource/get");
		client.send(fd);
		if (debug) console.log("storage: requesting object: " + obj);
	}

	var list = function(callback) {
		var client = new XMLHttpRequest();
		client.onreadystatechange = function() {
			if (debug) console.log("storage: readyState: " + this.readyState);
			if (this.readyState == this.DONE) {
				if (debug) console.log("storage: aquired object list");
				var data = JSON.parse(this.responseText);
				callback(data);
			}
		};
		client.open("GET","/resource/list");
		client.send();
		if (debug) console.log("storage: requesting object list");
	}

	var store = function(obj, rev, data) {
		var client = new XMLHttpRequest();
		var fd = new FormData();
		fd.append("obj", obj);
		if (rev !== undefined) fd.append("rev", rev);
		fd.append("data", data);
		client.open("POST", "/resource/add");
		if (debug) console.log("storage: sending: \"" + obj + "\": " + data);
		client.send(fd);
	}

	var get_username = function() {
		return session.username;
	}

	return {
		register :register,
		login: login,
		logout: logout,
		load: load, 
		list: list,
		store: store,
		get_username: get_username
	};

}();
