
define(["widget/newcadview", "widget/cadview/uicomponent", "mustache/mustache"],
function(NewCadView, uiComponent, Mustache) {
return function(parentNode, args) {

	// Object member objects/variables

	this.view = new NewCadView(parentNode, args);
	this.surface = this.view.surface;
	this.mustache = new Mustache(this.surface);
	this.objects = [];
	this.objectsUnderMouse = [];
	this.selectedObjects = [];


	var surface = this.surface;
	var objects = this.objects;
	var mustache = this.mustache;
	var mousePosition = {x: 0, y: 0};
	sur = surface;

	var deleteObject = function(obj) {
		var i = objects.indexOf(obj);
		if(i != -1) {
			objects.splice(i,1);
		}
	}

	var addComponentMode = function(resourceName) {
		var component = new uiComponent({shape: resourceName, pos: {x: mousePosition.x, y: mousePosition.y}});
		var moveComponent = function(event) {
				component.pos = {x: event.offsetX, y: event.offsetY};
				invalidate();
			}
			var placeComponent = function(event) {
				if(event.which == 1) {
					component = new uiComponent({shape: resourceName, pos: {x: mousePosition.x, y: mousePosition.y}});
					objects.push(component);
					invalidate();
				}
			}
			var enable = function() {
				objects.push(component);
				component.pos = {x: mousePosition.x, y: mousePosition.y};
				
				surface.addEventListener('mousemove', moveComponent, false);
				surface.addEventListener('click', placeComponent, false);
				surface.oncontextmenu = function(event) {
					component.angle += Math.PI*0.5;
					component.angle %= Math.PI*2.0;
					invalidate();
					return false;
				}
				invalidate();
			}
			var unregisterEvents = function() {
				surface.removeEventListener('mousemove', moveComponent, false);
				surface.oncontextmenu = function(event) {return false;}
				surface.removeEventListener('click', placeComponent, false);
			}
			var cancel = function() {
				deleteObject(component);
				unregisterEvents();
			}

			return {
				"cancel": cancel,
				"enable": enable
			}
		}

		var addResistorMode = new addComponentMode("schematic/resistor");
		var addCapacitorMode = new addComponentMode("schematic/capacitor");
		var addInductorMode = new addComponentMode("schematic/inductor");
		var addGroundMode = new addComponentMode("schematic/ground");

		var addWireMode = new function() {
			var fromPos = {x: 0, y: 0};
			var toPos = {x: 0, y: 0};
			var first = true;

			var wireSegment = new uiComponent({shape: "schematic/wiresegment", shapeParameters:{from: fromPos, to: toPos}});

			var placeWireSegment = function(event) {
				if(event.which == 1) {
					if(first) {
						fromPos.x = mousePosition.x;
						fromPos.y = mousePosition.y;
						toPos.x = mousePosition.x;
						toPos.y = mousePosition.y;
						objects.push(wireSegment);
						first = false;
					} else {
						toPos = {x: mousePosition.x, y: mousePosition.y};
						fromPos = {x: mousePosition.x, y: mousePosition.y};
						wireSegment = new uiComponent({shape: "schematic/wiresegment", shapeParameters: {from: fromPos, to: toPos}});
						objects.push(wireSegment);
					}
				}
			}

			var moveWire = function(event) {
				toPos.x = event.offsetX;
				toPos.y = event.offsetY;
				invalidate();
			}

			var enable = function() {
				surface.addEventListener('click', placeWireSegment, false);
				surface.addEventListener('mousemove', moveWire, false);
				invalidate();
			}
			var unregisterEvents = function() {
				surface.removeEventListener('click', placeWireSegment, false);
				surface.removeEventListener('mousemove', moveWire, false);
			}
			var cancel = function() {
				deleteObject(wireSegment);
				unregisterEvents();
				first = true;
			}


			return {
				"enable": enable,
				"cancel": cancel
			}
		}


		var idleMode = new function() {
			var enable = function() {
				surface.onclick = function(event) {}
				surface.oncontextmenu = function(event) {}
				surface.onmousemove = function(event) {}
				invalidate();
			}
			var cancel = function() {

			}

			return {
				"enable": enable,
				"cancel": cancel
			}
		};

		var currentEditorMode = idleMode;

		var setEditorMode = function(mode) {
			if(currentEditorMode == mode) return;
			if(currentEditorMode.cancel) currentEditorMode.cancel();
			currentEditorMode = mode;
			mode.enable();
		}

		var handleKeypress = function(event) {
			switch(event.keyCode) {

			// C button to add capacitor
			case 99:
				setEditorMode(addCapacitorMode);
				break;
			case 103:
				setEditorMode(addGroundMode);
				break;
			case 105:
				setEditorMode(addInductorMode);
				break;
			case 114:
				setEditorMode(addResistorMode);
				break;
			case 119:
				setEditorMode(addWireMode);
				break;
			default:
				console.log("key: " + event.keyCode);
				break;
			}
		}

		var logMousePosition = function(event) {
			mousePosition.x = event.offsetX;
			mousePosition.y = event.offsetY;
		}
		surface.addEventListener('mousemove', logMousePosition, false);

		document.onkeypress = handleKeypress;
		document.onkeyup = function(event) {
			if(event.keyCode == 27) {
				setEditorMode(idleMode);
			}
		}

		// Object functions

		var invalidate = function() {
			this.mustache.clearCanvas();
			objects.forEach(function(element) {
				this.mustache.drawShape(element);
			}.bind(this));
		}.bind(this);

		// Object events

		var scale = 1;

		/*
		this.surface.onmousemove = function(event) {
			var requestInvalidation = true;
			this.objectsUnderMouse.forEach(function(element) {
				if(!this.mustache.isPointInShape({x: event.offsetX, y: event.offsetY}, element)) {
					element.onMouseOut();
					requestInvalidation = true;
				}
			}.bind(this));
			this.objectsUnderMouse = [];
			this.objects.forEach(function(element) {
				if(this.mustache.isPointInShape({x: event.offsetX, y: event.offsetY}, element)) {
					this.objectsUnderMouse.push(element);
					element.onMouseOver();
					console.log("mouse!");
					requestInvalidation = true;
				}
			}.bind(this));
			if(requestInvalidation) this.invalidate();
		}.bind(this);
		*/

		/*
		this.surface.onclick = function(event) {
			ev = event;
			if(event.ctrlKey) {
				this.objectsUnderMouse.forEach(function(underMouseObj) {
					var objectAlreadySelected = false;
					this.selectedObjects.forEach(function(selectedObj) {
						if(selectedObj == underMouseObj) {
							objectAlreadySelected = true;
						}
						else {
							this.selectedObjects.push(underMouseObj);
						}
					}.bind(this));
					underMouseObj.select();
				}.bind(this));
			}
			else {
				this.selectedObjects.forEach(function(element) {
					element.deselect();
				}.bind(this));
				this.selectedObjects = [];
				this.objectsUnderMouse.forEach(function(element) {
					this.selectedObjects.push(element);
					element.select();
				}.bind(this));
			}
			this.invalidate();
		}.bind(this);
		*/

		this.view.resizeEvents.push(function(ev) {
			invalidate();
		}.bind(this));


		// initial redraw
		invalidate();
	};
});

