
define(["widget/cadviewmixin", "widget/gridlayermixin", "editor/schematic/schematicmodes", "editor/editors", "simulator",
		"editor/schematic/componentlayer", "resources/pin", "resources/interfaceendpoint", "resources/resistor", "resources/opamp", "resources/module",
		"resources/ground", "resources/capacitor", "resources/diode", "resources/supply", "widget/cadview/uicomponentmixin",
		"editor/schematicviewmixin", "widget/cadcomponent", "resources/circle", "resources/wiresegment", "resources/interfacesegment", "resources/text", "resources/line",
		"resources/wire", "resources/interface", "api", "widget/cadvieweditormodemixin", "editor/texteditor", "editor/schematiceditormenu", "dijit/Dialog"],
function(CadView, GridLayer, SchematicModes, editors, Simulator, ComponentLayer, Pin, InterfaceEndpoint, Resistor, OpAmp, Module, Ground, Capacitor, Diode, Supply, uiComponent, SchematicView, CadComponent,
	Circle, WireSegment, InterfaceSegment, Text, Line, Wire, Interface, API, CadViewEditorMode, TextEditor, schematicEditorMenu, Dialog) {
	var NSchematic = function(parentNode, args) {
		var nSchematic = this;
		this.parentNode = parentNode;
		this.title = args.title || "new NSchematic";
		this.selectedObjects = [];
		this.editorMode = undefined;

		this.netList = [];
		this.branchList = [];
		this.wireList = [];
		this.interfaceList = [];

		// Mixin CadView, GridLayer and SchematicView
		CadView.call(this);
		GridLayer.call(this);
		ComponentLayer.call(this);
		SchematicView.call(this);

		listBranches = function() {
			nSchematic.branchList.forEach(function(branch) {
				console.log(branch.type);
			});
		}
		listNodes = function() {
			console.log(nSchematic.wireList.length);
			nodes = nSchematic.wireList;
		}
		this.simulator = new Simulator({view: nSchematic});

		gen = function() {
			nSchematic.simulator.generateMatrices(nSchematic.wireList, nSchematic.branchList);
		}

		var pinActionPoint = function(parent) {
			var newActionArea = nSchematic.createActionArea({
				type: "pinActionPoint",
				shape: Circle,
				pos: parent.connectionPoint.globalPos,
				objects: [],
				onMouseOver: function(event) {
					//console.log("over pin!");
				},
				onMouseOut: function(event) {

				},
				onMouseDown: function(event) {

				},
				onMouseUp: function(event) {

				},
				onCreate: function(netName) {

				},
				onConnect: function(wireSegment) {
					var component = parent.parent;
					if(!wireSegment.wire.addEndpoint(parent)) return;

					if(component.pins.length == 2 && (component.type == "resistor" || component.type == "capacitor" || component.type == "inductor" || component.type == "supply")) {
						if(component.pins[0].wire != undefined && component.pins[1].wire != undefined) {
							var newBranch = {from: component.pins[0].wire, to: component.pins[1].wire, value: component.value, type: component.type};
							if(!component.value) newBranch.value = 1;
							nSchematic.branchList.push(newBranch);
						}
					}
					else if(component.type == "ground") {
						nSchematic.ground = wireSegment.wire;
						wireSegment.wire.potential = "ground";
						console.log("ground!");
						gnd = nSchematic.ground;
					}
				},
				onDisconnect: function(wireSegment) {
					console.log("disconnected from pin!");
				}
			});

			parent.connectionPoint.actionPoint = newActionArea;

			return newActionArea;
		}

		var interfaceEndpointActionPoint = function(parent) {
			var newActionArea = nSchematic.createActionArea({
				type: "interfaceEndpointActionPoint",
				shape: Circle,
				pos: parent.connectionPoint.globalPos,
				objects: [],
				onMouseOver: function(event) {
					//console.log("over interfaceEndpoint!");
				},
				onMouseOut: function(event) {

				},
				onMouseDown: function(event) {
				},
				onMouseUp: function(event) {

				},
				onCreate: function(netName) {

				},
				onConnect: function(interfaceSegment) {
					console.log("connected to interfaceEndpoint!");
				},
				onDisconnect: function(interfaceSegment) {
					console.log("disconnected from interfaceEndpoint!");
				}
			});

			parent.connectionPoint.actionPoint = newActionArea;
			actionarea = newActionArea;

			return newActionArea;
		}

		var wireActionPoint = function(position) {
			var newActionArea = nSchematic.createActionArea({
				type: "wireActionPoint",
				shape: Circle,
				pos: position,
				objects: [],
				onMouseOver: function(event) {
					//console.log("over interface!");
					if(!window.dbgAA || dbgAA != newActionArea) {
						dbgAA = newActionArea;
						//console.log("objects connected to action area: " + newActionArea.objects.length);
						//console.log("endpoints: " + newActionArea.wire.endpoints.length);
						console.log("node index: " + nSchematic.wireList.indexOf(newActionArea.wire));
					}
				},
				onMouseOut: function(event) {
					//console.log("out from wire!");
				},
				onMouseDown: function(event) {
					console.log("number of endpoints: " + newActionArea.parent);

				},
				onMouseUp: function(event) {

				},
				onCreate: function(netName) {
					console.log("created!");
				},
				onConnect: function(wireSegment) {
					console.log("connected!");
				},
				onDisconnect: function(wireSegment) {
					console.log("disconnected!");
				}
			});
			return newActionArea;
		}

		var interfaceActionPoint = function(position) {
			var newActionArea = nSchematic.createActionArea({
				type: "interfaceActionPoint",
				shape: Circle,
				pos: position,
				objects: [],
				onMouseOver: function(event) {
					//console.log("over interface!");
					if(!window.dbgAA || dbgAA != newActionArea) {
						dbgAA = newActionArea;
						console.log("objects connected to action area: " + newActionArea.objects.length);
					}
				},
				onMouseOut: function(event) {
					//console.log("out from interface!");
				},
				onMouseDown: function(event) {

				},
				onDoubleClick: function() {
					/*
					if(!newActionArea.interface.subschematic) {
						newActionArea.interface.subschematic = API.openTab("schematic", {title: "my interface"});
						newActionArea.interface.subschematic.parentModule = newActionArea.interface;
						newActionArea.interface.subschematic.onClose.push(function() {
							newActionArea.interface.subschematic = undefined;
						});
					}
					newActionArea.interface.subschematic.show();
					*/
					if(newActionArea.interface.edit) newActionArea.interface.edit();
				},
				onMouseUp: function(event) {

				},
				onCreate: function(netName) {

				},
				onConnect: function(interfaceSegment) {
					console.log("connected!");
				},
				onDisconnect: function(interfaceSegment) {
					console.log("disconnected!");
				}
			});
			return newActionArea;
		}

		this.createPin = function(args) {
			var newPin = new CadComponent(args);
			Pin.call(newPin, args);
			//uiComponent.call(newPin);
			newPin.connectionPoint.globalPos = {x: 0, y: 0};
			newPin.connectionPoint.globalAngle = 0;

			var newActionPoint = new pinActionPoint(newPin);
			newPin.actionAreas = [newActionPoint];

			nSchematic.netList.push(newPin);

			return newPin;
		}
		this.createInterfaceEndpoint = function(args) {
			var newInterfaceEndpoint = new CadComponent(args);
			InterfaceEndpoint.call(newInterfaceEndpoint, args);
			//uiComponent.call(newInterfaceEndpoint);
			newInterfaceEndpoint.connectionPoint.globalPos = {x: 0, y: 0};
			newInterfaceEndpoint.connectionPoint.globalAngle = 0;

			var newActionPoint = new interfaceEndpointActionPoint(newInterfaceEndpoint);
			newInterfaceEndpoint.actionAreas = [newActionPoint];

			return newInterfaceEndpoint;
		}
		this.createWireSegment = function(point1, point2) {
			var newWireSegment = nSchematic.createComponent(WireSegment, {parent: nSchematic, points: [point1, point2]});
			newWireSegment.actionArea = [];
			return newWireSegment;
		}
		this.createWire = function(points) {
			var newWire = nSchematic.createComponent(Wire, {parent: nSchematic, points: points});

			return newWire;
		}
		this.createInterfaceSegment = function(point1, point2) {
			var newInterfaceSegment = nSchematic.createComponent(InterfaceSegment, {parent: nSchematic, points: [point1, point2]});
			newInterfaceSegment.actionArea = [];
			return newInterfaceSegment;
		}
		this.createInterface = function(points) {
			var newInterface = nSchematic.createComponent(Interface, {parent: nSchematic, points: points});
			nSchematic.netList.push(newInterface.netList);


			return newInterface;
		}

		this.updateNetList = function() {

		}

		this.editorModes.createWireMode = new function() {
			this.newWire = undefined;
			this.newWireSegment = undefined;
			var createWireMode = this;

			CadViewEditorMode.call(this, nSchematic);

			this.onEnterMode = function(parameters) {
				if(!parameters) parameters = {};
				if(parameters.startPoint) {
					var currentActionArea = parameters.startPoint;
					this.newWire = nSchematic.createWire();
					var pos = nSchematic.cursorPosition;
					this.newWireSegment = nSchematic.createWireSegment(currentActionArea, pos);
					this.newWire.addWireSegment(this.newWireSegment);
				}
			}
			this.onLeaveMode = function() {
				if(this.newWireSegment) {
					if(createWireMode.newWire.shape.length == 1) {
						createWireMode.newWireSegment.actionArea.forEach(function(actionArea) {
							if(actionArea.objects.length == 1) {
								nSchematic.removeActionArea(actionArea);
							}
						});
					}
					delete this.newWireSegment.path;
					delete this.newWireSegment;
					createWireMode.newWireSegment = undefined;
					//nSchematic.removeObject(createWireMode.newWireSegment);
				}
				this.newWire = undefined;

				console.log("number of action areas: " + nSchematic.actionAreas.length);

				nSchematic.redraw();
			}
			this.onMouseMove = function(event) {
				if(nSchematic.objectsUnderMouse.length > 0) {
					if(nSchematic.objectsUnderMouse[0].onMouseOver) {
						nSchematic.objectsUnderMouse[0].onMouseOver();
					}
				}
			}
			this.onMouseDown = function(event) {
				var currentActionArea = undefined;
				var newActionAreaCreated = false;
				if(nSchematic.actionAreasUnderMouse.length > 0) {
					currentActionArea = nSchematic.actionAreasUnderMouse[0];
					if(currentActionArea.wire) {
						currentActionArea.wire.mergeWire(createWireMode.newWire);
						createWireMode.newWire = currentActionArea.wire;
					}
				} else {
					currentActionArea = new wireActionPoint({x: nSchematic.cursorPosition.x, y: nSchematic.cursorPosition.y});
					newActionAreaCreated = true;
				}
				if(!this.newWireSegment) {
					if(!createWireMode.newWire) {
						this.newWire = nSchematic.createWire();
					}
					var pos = currentActionArea.pos;
					this.newWireSegment = nSchematic.createWireSegment(pos, nSchematic.cursorPosition);
					this.newWireSegment.actionArea[0] = currentActionArea;
					this.newWire.addWireSegment(this.newWireSegment);
				}
				else {
					this.newWireSegment.points[1] = currentActionArea.pos;
					this.newWireSegment.actionArea[1] = currentActionArea;
					this.newWireSegment.actionArea[0].objects.push(this.newWireSegment);
					var firstActionArea = createWireMode.newWireSegment.actionArea[0];
					if(firstActionArea && firstActionArea.objects.length > 2 && firstActionArea.junction == undefined) {
						console.log("first action area!");
						var newJunction = nSchematic.createComponent(Circle, {pos: firstActionArea.pos, fillStyle: {value: "blue"}, radius: {value: 3}});
						firstActionArea.junction = newJunction;
					}
					this.newWireSegment.actionArea[1].objects.push(this.newWireSegment);
					var secondActionArea = createWireMode.newWireSegment.actionArea[1];
					if(secondActionArea && secondActionArea.objects.length > 2 && secondActionArea.junction == undefined) {
						console.log("second action area!");
						var newJunction = nSchematic.createComponent(Circle, {pos: secondActionArea.pos, fillStyle: {value: "blue"}, radius: {value: 3}});
						secondActionArea.junction = newJunction;
					}
					var pos = nSchematic.cursorPosition;
					this.newWireSegment = nSchematic.createWireSegment(currentActionArea.pos, pos);
					this.newWireSegment.actionArea[0] = currentActionArea;
					this.newWire.addWireSegment(this.newWireSegment);
				}
				console.log("objects connected to action area: " + currentActionArea.objects.length);
				currentActionArea.wire = createWireMode.newWire;
				if(currentActionArea.onConnect && !newActionAreaCreated) currentActionArea.onConnect(createWireMode.newWireSegment);

				if(nSchematic.objectsUnderMouse.length > 0) {
					if(nSchematic.objectsUnderMouse[0].onMouseDown) {
						nSchematic.objectsUnderMouse[0].onMouseDown(event);
					}
				}
			}
		}

		this.editorModes.createInterfaceMode = new function() {
			this.newInterface = undefined;
			this.newInterfaceSegment = undefined;
			var createInterfaceMode = this;

			CadViewEditorMode.call(this, nSchematic);

			this.onEnterMode = function(parameters) {
				if(!parameters) parameters = {};
				if(parameters.startPoint) {
					var currentActionArea = parameters.startPoint;
					this.newInterface = nSchematic.createInterface();
					var pos = nSchematic.cursorPosition;
					this.newInterfaceSegment = nSchematic.createInterfaceSegment(currentActionArea, pos);
					this.newInterface.addInterfaceSegment(this.newInterfaceSegment);
				}
			}
			this.onLeaveMode = function() {
				if(this.newInterfaceSegment) {
					if(createInterfaceMode.newInterface.shape.length == 1) {
						createInterfaceMode.newInterfaceSegment.actionArea.forEach(function(actionArea) {
							if(actionArea.objects.length == 1) {
								nSchematic.removeActionArea(actionArea);
							}
						});
					}
					delete this.newInterfaceSegment.path;
					delete this.newInterfaceSegment;
					createInterfaceMode.newInterfaceSegment = undefined;
					//nSchematic.removeObject(createInterfaceMode.newInterfaceSegment);
				}
				this.newInterface = undefined;

				console.log("number of action areas: " + nSchematic.actionAreas.length);

				nSchematic.redraw();
			}
			this.onMouseMove = function(event) {
				if(nSchematic.objectsUnderMouse.length > 0) {
					if(nSchematic.objectsUnderMouse[0].onMouseOver) {
						nSchematic.objectsUnderMouse[0].onMouseOver();
					}
				}
			}
			this.onMouseDown = function(event) {
				var currentActionArea = undefined;
				var newActionAreaCreated = false;
				if(nSchematic.actionAreasUnderMouse.length > 0) {
					currentActionArea = nSchematic.actionAreasUnderMouse[0];
					if(currentActionArea.interface) {
						createInterfaceMode.newInterface = currentActionArea.interface;
					}
				} else {
					currentActionArea = new interfaceActionPoint({x: nSchematic.cursorPosition.x, y: nSchematic.cursorPosition.y});
					newActionAreaCreated = true;
				}
				if(!this.newInterfaceSegment) {
					if(!createInterfaceMode.newInterface) {
						createInterfaceMode.newInterface = nSchematic.createInterface();
					}
					var pos = currentActionArea.pos;
					this.newInterfaceSegment = nSchematic.createInterfaceSegment(pos, nSchematic.cursorPosition);
					this.newInterfaceSegment.actionArea[0] = currentActionArea;
					this.newInterface.addInterfaceSegment(this.newInterfaceSegment);
				}
				else {
					this.newInterfaceSegment.points[1] = currentActionArea.pos;
					this.newInterfaceSegment.actionArea[1] = currentActionArea;
					//currentActionArea.objects.push(this.newInterfaceSegment);
					this.newInterfaceSegment.actionArea[0].objects.push(this.newInterfaceSegment);
					var firstActionArea = this.newInterfaceSegment.actionArea[0];
					if(firstActionArea && firstActionArea.objects.length > 2 && firstActionArea.junction == undefined) {
						var newJunction = nSchematic.createComponent(Circle, {pos: firstActionArea.pos, fillStyle: {value: "green"}, radius: {value:5}});
						firstActionArea.junction = newJunction;
					}
					this.newInterfaceSegment.actionArea[1].objects.push(this.newInterfaceSegment);
					var secondActionArea = this.newInterfaceSegment.actionArea[1];
					if(secondActionArea && secondActionArea.objects.length > 2 && secondActionArea.junction == undefined) {
						var newJunction = nSchematic.createComponent(Circle, {pos: secondActionArea.pos, fillStyle: {value: "green"}, radius: {value:5}});
						secondActionArea.junction = newJunction;
					}
					var pos = nSchematic.cursorPosition;
					this.newInterfaceSegment = nSchematic.createInterfaceSegment(currentActionArea.pos, pos);
					this.newInterfaceSegment.actionArea[0] = currentActionArea;
					this.newInterface.addInterfaceSegment(this.newInterfaceSegment);
				}
				//currentActionArea.objects.push(createInterfaceMode.newInterfaceSegment);
				console.log("added object to current action area");
				currentActionArea.interface = createInterfaceMode.newInterface;

				if(currentActionArea.onConnect && !newActionAreaCreated) currentActionArea.onConnect(createInterfaceMode.newInterfaceSegment);

				if(nSchematic.objectsUnderMouse.length > 0) {
					if(nSchematic.objectsUnderMouse[0].onMouseDown) {
						nSchematic.objectsUnderMouse[0].onMouseDown(event);
					}
				}
			}
			this.keyBindings[9] = function() {
				if(createInterfaceMode.newInterface) {
					nSchematic.suppressKeypress = true;
					//componentConfigDialog.set("content", currentMode.newComponent.configDialog);
					//nSchematic.showDialog(createInterfaceMode.newInterface.configDialog.container);
					if(createInterfaceMode.newInterface.config) createInterfaceMode.newInterface.config();
				}
			}
		}

		var placeComponentMode = function(component) {
			var currentMode = this;
			if(!currentMode.componentParameters) currentMode.componentParameters = {};
			if(!currentMode.componentParameters.pos) currentMode.componentParameters.pos = nSchematic.cursorPosition;
			if(!currentMode.componentParameters.angle) currentMode.componentParameters.angle = currentMode.angle;
			this.newComponent;
			this.angle;
			var componentConfigContainer = document.createElement("div");
			var componentConfigContent = document.createElement("div");
			componentConfigContainer.appendChild(componentConfigContent);
			this.closeConfigDialog = function() {
				componentConfigDialog.hide();
				nSchematic.redraw();
			}
			var componentConfigDialog = new Dialog({
				title: "",
				content: componentConfigContainer,
				onShow: function() {
				},
				onHide: function() {
					nSchematic.suppressKeypress = false;
					while(componentConfigContent.childElementCount) {
						componentConfigContent.removeChild(componentConfigContent.firstChild);
					}
				}
			});
			CadViewEditorMode.call(this, nSchematic);
			this.onEnterMode = function() {
				currentMode.newComponent = nSchematic.createComponent(component, currentMode.componentParameters);
				nSchematic.redraw();
			}
			this.onLeaveMode = function() {
				nSchematic.componentLayer.removeObject(currentMode.newComponent);
				nSchematic.removeObject(currentMode.newComponent);
				currentMode.newComponent = undefined;
				nSchematic.redraw();
			}
			this.onMouseDown = function(event) {
				switch(event.which) {
				case 1:
					currentMode.newComponent.pos = {x: nSchematic.cursorPosition.x, y: nSchematic.cursorPosition.y};
					if(currentMode.onPlaceComponent) currentMode.onPlaceComponent(currentMode.newComponent);
					currentMode.newComponent = nSchematic.createComponent(component, currentMode.componentParameters);
					nSchematic.redraw();
					break;
				case 3:
					currentMode.newComponent.angle = (currentMode.newComponent.angle + Math.PI/2);
					nSchematic.updateGlobalPositions();
					nSchematic.redraw();
					break;
				}
			}
			this.keyBindings[9] = function() {
				if(currentMode.newComponent.configDialog) {
					nSchematic.suppressKeypress = true;
					//componentConfigDialog.set("content", currentMode.newComponent.configDialog);
					componentConfigContent.appendChild(currentMode.newComponent.configDialog.container);
					componentConfigDialog.show();
				}
			}
		}

		this.editorModes.placeResistorMode = new function() {
			placeComponentMode.call(this, Resistor);
		}

		this.editorModes.placeOpAmpMode = new function() {
			placeComponentMode.call(this, OpAmp);
		}
		this.editorModes.placeCapacitorMode = new function() {
			placeComponentMode.call(this, Capacitor);
		}
		this.editorModes.placeDiodeMode = new function() {
			placeComponentMode.call(this, Diode);
		}
		this.editorModes.placeGroundMode = new function() {
			placeComponentMode.call(this, Ground);
		}
		this.editorModes.placeSupplyMode = new function() {
			placeComponentMode.call(this, Supply);
		}
		this.editorModes.placeModuleMode = new function() {
			var placeModuleMode = this;
			var moduleNameContainer = document.createElement("div");
			var moduleNameTextfield = document.createElement("input");
			moduleNameTextfield.type = "text";
			moduleNameContainer.appendChild(moduleNameTextfield);
			var myDialog = new Dialog({
				title: "module name",
				content: moduleNameContainer,
				onHide: function() {
					nSchematic.suppressKeypress = false;
					moduleNameTextfield.blur();
				}
			});
			moduleNameTextfield.onkeypress = function(event) {
				if(event.which == 13 && moduleNameTextfield.value.length > 0) {
					placeModuleMode.componentParameters.name.value = moduleNameTextfield.value;
					myDialog.hide();
					nSchematic.redraw();
				}
			}

			this.onPlaceComponent = function(component) {

			}
			placeComponentMode.call(this, Module);
		}


		this.createComponent = function(component, args) {
			args.view = nSchematic;
			var newComponent = new CadComponent(args);
			component.call(newComponent, args);
			//uiComponent.call(newComponent);
			nSchematic.objects.push(newComponent);
			nSchematic.componentLayer.objects.push(newComponent);
			nSchematic.updateGlobalPositions();
			return newComponent;
		}
		/*
		this.removeComponent = function(component) {
			if(component.pins) {
				if(Array.isArray(component.pins)) {
					component.pins.forEach(function(pin) {
						nSchematic.removeObject(pin);
					});
				}
				else {

					nSchematic.removeObject(pin);
				}
			}
			nSchematic.removeObject(component);
		}
		*/

		this.onShow.push(function() {
		});

		/*
		var inputResistor = this.createComponent(Resistor,{pos:{x:275, y:50}, angle:Math.PI/2});
		var gainResistor = this.createComponent(Resistor,{pos:{x:450, y:175}, angle: Math.PI/2});
		var drainResistor = this.createComponent(Resistor, {pos:{x: 350, y: 250}, angle: 0});
		var myOpAmp = this.createComponent(OpAmp,{pos: {x: 450, y: 75}, angle: Math.PI*0.0});
		var noninvertingWire = this.createWire([inputResistor.pins[0].connectionPoint.globalPos, myOpAmp.noninverting.connectionPoint.globalPos]);
		var invertingWire = this.createWire([myOpAmp.inverting.connectionPoint.globalPos, {x: 350, y: 100}, {x: 350, y: 200}]);
		//var invertingWire2 = this.createWire([{x: 350, y: 175}, {x: 400, y: 175}]);
		var invertingWire3 = this.createWire([{x: 500, y: 175}, {x: 550, y: 175}, {x: 550, y: 75}]);
		var outputWire = this.createWire([myOpAmp.output.connectionPoint.globalPos, {x: 600, y: 75}]);
		var groundingWire = this.createWire([{x: 350, y: 300}, {x: 350, y: 325}]);
		*/

		/*
		var myOpAmp = this.createComponent(OpAmp,{pos: {x: 450, y: 175}, angle: Math.PI*0.0});
		window.setInterval(function() {
			myOpAmp.angle += Math.PI*0.025;
			//inputResistor.angle -= Math.PI*0.05;
			nSchematic.updateGlobalPositions();
			nSchematic.redraw();
		}, 100);
		*/

		//var newRes = this.createComponent(Resistor,{pos:nSchematic.cursorPosition, angle:Math.PI/2});
		//var myOpAmp2 = this.createComponent(OpAmp,{pos:{x:650, y:100}});

		/*
		var myText = this.createComponent(Text, {text: {value: "lalala"}, pos:{x:300,y:300}, alwaysUpright: true, angle: 0});
		window.setInterval(function() {
			myText.angle -= Math.PI*0.025;
			//inputResistor.angle -= Math.PI*0.05;
			nSchematic.updateGlobalPositions();
			nSchematic.redraw();
		}, 100);
		*/

		var cursor = this.createComponent(Text, {text: "+", pos: nSchematic.cursorPosition, align: "center"});
	
		this.updateGlobalPositions();

		(function addKeyBindings() {
			nSchematic.editorModes.idleMode.keyBindings["W".charCodeAt(0)] = function() {
				nSchematic.setEditorMode(nSchematic.editorModes.createWireMode);
			}
			nSchematic.editorModes.idleMode.keyBindings["I".charCodeAt(0)] = function() {
				nSchematic.setEditorMode(nSchematic.editorModes.createInterfaceMode);
			}
			nSchematic.editorModes.idleMode.keyBindings["R".charCodeAt(0)] = function() {
				nSchematic.setEditorMode(nSchematic.editorModes.placeResistorMode);
			}
			nSchematic.editorModes.idleMode.keyBindings["O".charCodeAt(0)] = function() {
				nSchematic.setEditorMode(nSchematic.editorModes.placeOpAmpMode);
			}
			nSchematic.editorModes.idleMode.keyBindings["M".charCodeAt(0)] = function() {
				nSchematic.setEditorMode(nSchematic.editorModes.placeModuleMode);
			}
			nSchematic.editorModes.idleMode.keyBindings["G".charCodeAt(0)] = function() {
				nSchematic.setEditorMode(nSchematic.editorModes.placeGroundMode);
			}
			nSchematic.editorModes.idleMode.keyBindings["C".charCodeAt(0)] = function() {
				nSchematic.setEditorMode(nSchematic.editorModes.placeCapacitorMode);
			}
			nSchematic.editorModes.idleMode.keyBindings["D".charCodeAt(0)] = function() {
				nSchematic.setEditorMode(nSchematic.editorModes.placeDiodeMode);
			}
			nSchematic.editorModes.idleMode.keyBindings["S".charCodeAt(0)] = function() {
				nSchematic.setEditorMode(nSchematic.editorModes.placeSupplyMode);
			}

		})();

		this.init();
	}

	editors["schematic"] = NSchematic;

	return NSchematic;
});

