

define(["resources/text", "api", "dojo/text!resources/moduleconfig.html", "dojo/text!resources/modulepinnameconfig.html", "resources/circle"],
function(Text, API, ModuleConfig, ModulePinNameConfig, Circle) {
	return function(parameters) {
		var Module = this;
		if(parameters) {
			Module.pos = parameters.pos || {x: 0, y: 0};
			Module.angle = parameters.angle || 0;
			Module.view = parameters.view;
			Module.name = parameters.name;
		}
		else {
			Module.pos = {x: 0, y: 0};
			Module.angle = 0;
		}

		if(!this.name) this.name = {value:"new module"};

		this.width = 100;
		this.height = 100;
		this.ratio = this.height/this.width;

		this.type = "module";
		this.subschematic = undefined;

		this.boundingPath = [
			{
				type: "line",
				points: [
					{x: -((Module.width/2)+25), y: -((Module.height/2)+25)},
					{x: -((Module.width/2)+25), y:  ((Module.height/2)+25)},
					{x:  ((Module.width/2)+25), y:  ((Module.height/2)+25)},
					{x:  ((Module.width/2)+25), y: -((Module.height/2)+25)}
				]
			}
		];

		this.onDoubleClick = function(event) {
			if(!Module.subschematic) {
				Module.subschematic = API.openTab("schematic", {title: (Module.name)? Module.name.value: "new module"});
				Module.subschematic.parentModule = Module;
				Module.subschematic.onClose.push(function() {
					Module.subschematic = undefined;
				});
			}
			Module.subschematic.show();
		}

		var view = this.view;
		var newShape = {shape: undefined};
		var newShapePos = {x: 0, y: 0};
		var newShapeAngle = {value: 0};

		var moduleNameContainer = document.createElement("div");
		moduleNameContainer.innerHTML = ModuleConfig;
		var moduleConfigForm = moduleNameContainer.querySelector("#moduleConfigForm");
		var moduleNameInput = moduleNameContainer.querySelector("#moduleName");
		var cancelButton = moduleNameContainer.querySelector("#cancelButton");

		moduleNameInput.onfocus = function() {
			moduleNameInput.value = Module.name.value;
			moduleNameInput.select();
		}
		moduleConfigForm.onsubmit = function(event) {
			Module.name.value = moduleNameInput.value;
			view.currentEditorMode.closeConfigDialog();
			return false;
		}
		moduleNameInput.onkeypress = function(event) {

		}
		cancelButton.onclick = function() {
			view.currentEditorMode.closeConfigDialog();
		}
		//this.configDialog = moduleNameContainer;
		this.configDialog = {
			container: 	moduleNameContainer,
			form:		moduleConfigForm
		}
		var modulePinNameConfig = document.createElement("div");
		modulePinNameConfig.innerHTML = ModulePinNameConfig;
		var pinNameConfigForm = modulePinNameConfig.querySelector("#pinNameConfigForm");
		var pinNameInput = modulePinNameConfig.querySelector("#pinName");
		pinNameInput.onfocus = function() {
			if(newShape.shape && (newShape.shape.type == "pin" || newShape.shape.type == "interfaceendpoint")) {
				if(newShape.shape.name.value) {
					pinNameInput.value = newShape.shape.name.value;
				}
				else if(newShape.shape.text.value) {
					pinNameInput.value = newShape.shape.text.value;
				}
			}
		}
		pinNameConfigForm.onsubmit = function(event) {
			if(newShape.shape && (newShape.shape.type == "pin" || newShape.shape.type == "interfaceendpoint")) {
				if(newShape.shape.name.value) {
					newShape.shape.name.value = pinNameInput.value;
				}
				else {
					newShape.shape.text.value = pinNameInput.value;
				}
			}
			view.closeDialog();
			view.redraw();
		}

		this.onMouseOver = function() {
			if(view.currentEditorMode == view.editorModes.createWireMode || view.currentEditorMode == view.editorModes.createInterfaceMode) {
				newShapePos.x = view.cursorPosition.x-Module.pos.x;
				newShapePos.y = view.cursorPosition.y-Module.pos.y;


				// if cursor is in corner of module (0.002 added due to floating point error)
				if((Math.abs(newShapePos.x)-0.002 > Module.width/2) && (Math.abs(newShapePos.y)-0.002 > Module.height/2)) {
					Module.onMouseOut();
					return;
				}

				if(view.currentEditorMode.keyBindings[9] == undefined) {
					if(view.currentEditorMode == view.editorModes.createWireMode) {
						view.currentEditorMode.keyBindings[9] = function() {
							view.showDialog(modulePinNameConfig);
						}
					}
					/*
					else if(view.currentEditorMode == view.editorModes.createInterfaceMode) {
						view.currentEditorMode.keyBindings[9] = function() {
							view.showDialog(modulePinNameConfig);
						}
					}
					*/
				}

				// Snap to vertical lines of module
				if(Math.abs(newShapePos.x)-Math.abs(newShapePos.y)/Module.ratio > 0) {
					// Snap to left vertical line
					if(newShapePos.x < 0) {
						newShapePos.x = -Module.width/2;
						newShapeAngle.value = Math.PI/2;
					}
					// Snap to right vertical line
					else {
						newShapePos.x = Module.width/2;
						newShapeAngle.value = -Math.PI/2;
					}
				}
				// Snap to horizontal lines of module
				else {
					// Snap to top horizontal line
					if(newShapePos.y < 0) {
						newShapePos.y = -Module.height/2;
						newShapeAngle.value = Math.PI;
					}
					// Snap to bottom horizontal line
					else {
						newShapePos.y = Module.height/2;
						newShapeAngle.value = 0;
					}
				}
				
				if(newShape.shape == undefined) {
					if(view.currentEditorMode == view.editorModes.createWireMode) {
						var pinNameValue = Module.pins.length + 1;
						var pinName = {value: pinNameValue.toString()};
						newShape.shape = view.createPin({angle: newShapeAngle, pos: newShapePos, length: 25, parent: Module, text: pinName});
						newShape.shape.connectionPoint.actionPoint.onMouseDown = Module.onMouseDown;
					}
					else if(view.currentEditorMode == view.editorModes.createInterfaceMode) {
						var interfaceName = "interface";
						var interface = view.currentEditorMode.newInterface;
						if(interface) {
							interfaceName = interface.interfaceName;
						}
						newShape.shape = view.createInterfaceEndpoint({angle: newShapeAngle, pos: newShapePos, interface: interface, length: 25, parent: Module, name: {value: interfaceName}});
						newShape.shape.connectionPoint.actionPoint.onMouseDown = Module.onMouseDown;
					}
				}

				view.updateGlobalPositions();
				view.cursorPosition.x = newShape.shape.connectionPoint.actionPoint.pos.x;
				view.cursorPosition.y = newShape.shape.connectionPoint.actionPoint.pos.y;
			}
		}
		this.onMouseOut = function() {
			if(newShape.shape && (newShape.shape.type == "pin" || newShape.shape.type == "interfaceendpoint")) {
				console.log("deleted shape!");
				view.removeActionArea(newShape.shape.connectionPoint.actionPoint);
				view.removeObject(newShape.shape);
				newShape.shape = undefined;
			}
			if(view.currentEditorMode == view.editorModes.createWireMode/* || view.currentEditorMode == view.editorModes.createInterfaceMode*/) {
				view.currentEditorMode.keyBindings[9] = undefined;
			}
			view.updateGlobalPositions();
		}
		this.onMouseDown = function(event) {
			if(view.currentEditorMode == view.editorModes.createWireMode || view.currentEditorMode == view.editorModes.createInterfaceMode) {
				if(event.which == 1) {
					if(newShape.shape.type == "pin") {
						Module.pins.push(newShape.shape);
					}
					else if(newShape.shape.type == "interfaceendpoint") {
						Module.interfaces.push(newShape.shape);
						view.currentEditorMode.newInterface.addEndpoint(newShape.shape);
						var actionArea = view.currentEditorMode.newInterfaceSegment.actionArea[0];
						var newJunction = view.createComponent(Circle, {pos: actionArea.pos, fillStyle: {value: "green"}, radius: {value:7}});
					}
					newShape.shape.connectionPoint.actionPoint.onMouseDown = undefined;
					newShape.shape.pos = {x: newShapePos.x, y: newShapePos.y};
					newShape.shape.angle = {value: newShapeAngle.value};
					newShape.shape = undefined;
				}
			}
			else if(view.currentEditorMode == view.editorModes.idleMode) {
				if(event.which == 3) {

				}
			}
		}

		this.fillColor = {value: "#FFFF80"};
		this.select = function() {
			Module.fillColor.value = "#FF8080";
		}
		this.deselect = function() { 
			Module.fillColor.value = "#FFFF80";
		}

		this.rebuildModule = function() {
			
		}

		this.pins = [
			//view.createPin({angle:Math.PI, pos:{x:0,y:-50}, length:25, parent: Module}),
			//view.createPin({angle:0, pos:{x:0,y:50}, length:25, parent: Module})
		];
		this.interfaces = [

		];
		this.shape = [
			new function() {
				this.fillStyle = Module.fillColor;
				this.strokeStyle = "#000000";
				this.closed = true;
				this.path = [
					{
						type: "line",
						points: [
							{x: -(Module.width/2), y: -(Module.height/2)},
							{x: -(Module.width/2), y:  (Module.height/2)},
							{x:  (Module.width/2), y:  (Module.height/2)},
							{x:  (Module.width/2), y: -(Module.height/2)}
						]
					}
				]
			},
			new Text({text: Module.name, fixedAngle: true, align: "center", font: "10pt monospace"}),
			Module.pins,
			Module.interfaces,
			newShape
		]
	}
});

