

define(["resources/pin", "resources/resistor", "resources/capacitor"],
function(Pin, Resistor, Capacitor) {
	var resources = [];

	var getResource = function(resource) {
		var matchingResource = null;
		resources.forEach(function(res) {
			if(res.name == resource) {
				matchingResource = res;
			}
		});
		return matchingResource;
	}

	var resource = function(resourceName, parameters) {
		var loadedResource = loadResource(resourceName);
		return new loadedResource(parameters);
	}

	var loadResource = function(resourceName) {
		var loadedResource = null;
		if(loadedResource = getResource(resourceName)) {
			return loadedResource.value;
		}
		var newResourceEntry = null;
		if(resourceName == "schematic/pin") {
			newResourceEntry = {
				name: resourceName,
				value: Pin
			};
		}
		else if(resourceName == "schematic/resistor") {
			newResourceEntry = {
				name: resourceName,
				value: Resistor
			};
		}
		else if(resourceName == "schematic/module") {
			newResourceEntry = {
				name: resourceName,
				value: function(parameters) {
					this.angle = 0;
					if(parameters) {
						this.angle = parameters.angle;
					}
					this.shape = [
						new resource("schematic/pin", {angle: Math.PI, pos: {x: 0, y: -30}}),
						new resource("schematic/pin", {angle: 0, pos: {x: 0, y: 30}}),
						new function() {
							this.width = 60;
							this.height = 60;
							this.fillStyle = "#FFFF80";
							this.strokeStyle = "#000000";
							this.closed = true;
							if(parameters) {
								this.pos = parameters.pos;
								this.angle = 0;
								this.width = parameters.width;
								this.height = parameters.height;
								this.fillStyle = parameters.fillStyle;
							}
							this.path = [
								{
									type: "line",
									points: [
										{x: -this.width/2, y: -this.height/2},
										{x: -this.width/2, y: this.height/2},
										{x: this.width/2, y: this.height/2},
										{x: this.width/2, y: -this.height/2}
									]
								}
							]
						}
					];
				}
			};
		}
		else if(resourceName == "schematic/ground") {
			newResourceEntry = {
				name: resourceName,
				value: function(parameters) {
					this.angle = 0;
					if(parameters) {
						this.angle = parameters.angle || this.angle;
					}
					this.boundingShape = [
						{
							type: "line",
							points: [
								{x: -17,y: 0},
								{x: -17,y: 16},
								{x: 17,y: 16},
								{x: 17,y: 0}
							]
						}
					];
					this.shape = [
						new function() {
							this.shape = [
								new resource("schematic/pin", {angle: Math.PI, pos: {x: 0, y: 10}, length: 10}),
								new function() {
									this.strokeStyle = "000000";
									this.path = [
										{
											type: "line",
											points: [
												{x: -17, y: 10},
												{x: 17, y: 10}
											]
										},
										{
											type: "line",
											points: [
												{x: -12, y: 15},
												{x: 12, y: 15}
											]
										},
										{
											type: "line",
											points: [
												{x: -7, y: 20},
												{x: 7, y: 20}
											]
										}
									]
								}
							];
						}
					];
				}
			};

		}
		else if(resourceName == "schematic/capacitor") {
			newResourceEntry = {
				name: resourceName,
				value: Capacitor
			};
		}
		else if(resourceName == "schematic/inductor") {
			newResourceEntry = {
				name: resourceName,
				value: function(parameters) {
					this.angle = 0;
					if(parameters) {
						this.angle = parameters.angle || this.angle;
					}
					this.boundingShape = [
						{
							type: "line",
							points: [
								{x: -17,y: 0},
								{x: -17,y: 16},
								{x: 17,y: 16},
								{x: 17,y: 0}
							]
						}
					];
					this.shape = [
						new function() {
							this.strokeStyle = "#000000";
							this.shape = [
								new resource("schematic/pin", {pos:{x:0,y:20}}),
								new resource("schematic/pin", {pos:{x:0,y:-20}, angle:Math.PI}),
								new function() {
									this.pos = {x: 0, y: 0};
									this.strokeStyle = "#000000";
									this.path = [
										{
											type: "arc",
											x: 0,
											y: -15,
											r: 5,
											start: -Math.PI/2,
											end: Math.PI/2,
											ccw: false
										},
										{
											type: "arc",
											x: 0,
											y: -5,
											r: 5,
											start: -Math.PI/2,
											end: Math.PI/2,
											ccw: false
										},
										{
											type: "arc",
											x: 0,
											y: 5,
											r: 5,
											start: -Math.PI/2,
											end: Math.PI/2,
											ccw: false
										},
										{
											type: "arc",
											x: 0,
											y: 15,
											r: 5,
											start: -Math.PI/2,
											end: Math.PI/2,
											ccw: false
										}
									]
								}
							];
						}
					];
				}
			};
		}
		else if(resourceName == "schematic/wire") {

		}
		else {
			console.log("no valid resource: " + resourceName);
			return null;
		}
		resources.push(newResourceEntry);
		console.log("new resource: " + resourceName);
		return newResourceEntry.value;
	};

	return resource;
});
