define(["gui/menubar", "dijit/DropDownMenu", "dijit/MenuItem", "dijit/PopupMenuBarItem", /*"plot/plot",*/ "simulator/solver", "gui/tabarea", "dijit/layout/ContentPane", "dojo/dom-construct"],
function (menuBar, DropDownMenu, MenuItem, PopupMenuBarItem, /*plot,*/ irksolver, tabArea, pane, domc) {
return function(args) {
	if(args.view) this.view = args.view;

	var Simulator = this;

	var systemMatrices = {
		C: undefined,
		A_C: undefined,
		G: undefined,
		A_R: undefined,
		L: undefined,
		A_L: undefined,
		I: undefined,
		A_I: undefined,
		E: undefined,
		A_V: undefined
	}

	this.start = function () {
		var that = this;
		this.runToggle.set("onClick", function() {that.stop()});
		this.runToggle.set("label", "Stop");
		this.running = true;
		this.timer = setTimeout(function() {that.update();}, this.period);
		this.simData = [];
	}

	this.stop = function () {
		var that = this;
		this.running = false;
		this.runToggle.set("onClick", function(){that.start()});
		this.runToggle.set("label", "Start");
		//this.plotter.refresh();
		//this.cp.set('content', this.simData[0]);
	}

	this.update = function () {
		if (this.running === false) return;
		var that = this;
		var state = irksolver();
		irksolver();irksolver();irksolver();irksolver();irksolver();irksolver();irksolver();irksolver();irksolver();
		//console.log(state.time + ',' + state.value.elements[0]);
		//this.simData.push("[" + state.time + ", " + state.value.elements[0] + "]\n");
		this.cp.set('content', this.cp.get('content') + state.time + ", " + state.value.elements[0] + "<br/>");
		//this.plotter.addValue({value: 1000*state.elements[0], time: this.time}); 
		//this.plotter.refresh();

		if (state.time < 0.2) {
			this.timer = setTimeout(function() {that.update();}, this.period);
		} else {
			this.stop();
		}
	}

	this.generateMatrices = function(netlist, branchlist) {
		var groundNode = -1;
		for(var i = 0; i < netlist.length; i++) {
			if(netlist[i].potential == "ground") groundNode = i;
		}
		var numNodes = netlist.length;
		var createARMatrix = function() {
			var A_R = [];
			var admitances = [];
			branchlist.forEach(function(branch) {
				// AR matrix only concerns resistors, so skip all other components
				if(branch.type != "resistor") return;
				// If the pins of the resistor are short-circuited, skip the component
				if(branch.from == branch.to) return;
				var pin0Index = netlist.indexOf(branch.from);
				var pin1Index = netlist.indexOf(branch.to);
				if(pin0Index == -1 || pin1Index == -1) return;
				var column = new Array(numNodes);
				for(var i = 0; i < column.length; i++) {
					column[i] = 0;
				}
				column[pin0Index] = 1;
				column[pin1Index] = -1;
				if(groundNode != -1) column.splice(groundNode,1);
				A_R.push(column);
				admitances.push(1/(branch.value));
			});
			if(admitances.length == 0) return;
			var mat = $M(A_R);
			systemMatrices.A_R = mat.transpose();
			systemMatrices.G = Matrix.Diagonal(admitances);
		}
		var createACMatrix = function() {
			var A_C = [];
			var capacitances = [];
			branchlist.forEach(function(branch) {
				// AC matrix only concerns capacitors, so skip all other components
				if(branch.type != "capacitor") return;
				// If the pins of the capacitor are short-circuited, skip the component
				if(branch.from == branch.to) return;
				var pin0Index = netlist.indexOf(branch.from);
				var pin1Index = netlist.indexOf(branch.to);
				if(pin0Index == -1 || pin1Index == -1) return;
				var column = new Array(numNodes);
				for(var i = 0; i < column.length; i++) {
					column[i] = 0;
				}
				column[pin0Index] = 1;
				column[pin1Index] = -1;
				if(groundNode != -1) column.splice(groundNode,1);
				A_C.push(column);
				capacitances.push(branch.value);
			});
			if(capacitances.length == 0) return;
			var mat = $M(A_C);
			systemMatrices.A_C = mat.transpose();
			systemMatrices.C = Matrix.Diagonal(capacitances);
		}
		var createAVMatrix = function() {
			var A_V = [];
			var E = [];
			branchlist.forEach(function(branch) {
				// AR matrix only concerns supplies, so skip all other components
				if(branch.type != "supply") return;
				// If the pins of the supply are short-circuited, skip the component
				if(branch.from == branch.to) return;
				var pin0Index = netlist.indexOf(branch.from);
				var pin1Index = netlist.indexOf(branch.to);
				if(pin0Index == -1 || pin1Index == -1) return;
				var column = new Array(numNodes);
				for(var i = 0; i < column.length; i++) {
					column[i] = 0;
				}
				column[pin0Index] = 1;
				column[pin1Index] = -1;
				if(groundNode != -1) column.splice(groundNode,1);
				A_V.push(column);
				E.push(branch.value);
			});
			if(E.length == 0) return;
			systemMatrices.A_V = $M(A_V).transpose();
			systemMatrices.E = $V(E);
		}
		createARMatrix();
		createACMatrix();
		createAVMatrix();

		matrix = systemMatrices;
		/*
		var columns = [];
		createARMatrix().forEach(function(column) {
			columns.push(column);
		});
		createACMatrix().forEach(function(column) {
			columns.push(column);
		});
		createAVMatrix().forEach(function(column) {
			columns.push(column);
		});
		return columns;
		*/

	}

	var that = this;
	this.runToggle = new MenuItem({
		label: "Run",
		onClick: function() {that.start()}
	});

	var menu = new DropDownMenu({});
	menu.addChild(this.runToggle);
	menuBar.addChild(new PopupMenuBarItem({
		label: "Simulator",
		popup: menu
	}));

	/*
	this.plotter = new plot({
		closable: false,
		title: "Plot",
	});
	tabArea.addChild(this.plotter);*/

	this.running = false;
	this.time = 0;
	this.period = 10;

}});
