
define(["widget/surfaceviewmixin", "dijit/Dialog"],
function(SurfaceView, Dialog) {
	return function() {
		if(!this.surfaceViewMixin) SurfaceView.call(this);
		this.cadViewMixin = true;
		var cadView = this;
		this.actionAreas = [];
		this.interactionLayer = this.createLayer("interaction layer", 10000);
		this.suppressTab = true;

		this.snapping = {x: 25, y: 25};
		this.hasMouseFocus = false;

		this.currentEditorMode = undefined;

		this.editorModes = {};
		this.selectedObjects = [];

		this.editorModes.idleMode = new function() {
			this.keyBindings = new Array(256);
			var idleMode = this;
			
			var leftClick = function(event) {
				idleMode.onMouseMove(event);
				var activeObject;
				if(!event.ctrlKey) {
					cadView.selectedObjects.forEach(function(object) {
						if(object.deselect) object.deselect();
					});
					cadView.selectedObjects = [];
				}
				if(cadView.actionAreasUnderMouse.length > 0) {
					//cadView.selectedObjects.push(cadView.actionAreasUnderMouse[0]);
					console.log("actionArea under mouse");
				}
				else if(cadView.objectsUnderMouse.length > 0) {
					// If object isn't already selected, select it
					if(cadView.selectedObjects.indexOf(cadView.objectsUnderMouse[0]) == -1) {
						cadView.selectedObjects.push(cadView.objectsUnderMouse[0]);
						if(cadView.objectsUnderMouse[0].select) cadView.objectsUnderMouse[0].select();
					}
				}
				else if(!event.ctrlKey) {
					cadView.selectedObjects.forEach(function(object) {
						if(object.deselect) object.deselect();
					});
					cadView.selectedObjects = [];
				}

				console.log("selected objects: " + cadView.selectedObjects.length);

				cadView.redraw();

				//if(activeObject.onMouseDown) activeObject.onMouseDown(event);
			}
			var rightClick = function(event) {
				if(cadView.objectsUnderMouse.length > 0) {
					if(cadView.objectsUnderMouse[0].onMouseDown) {
						cadView.objectsUnderMouse[0].onMouseDown(event);
					}
				}
			}

			this.onMouseMove = function(event) {

			}
			this.onMouseOut = function(event) {
				cadView.hasMouseFocus = false;
				updateObjectsUnderMouse();
			}
			this.onMouseOver = function(event) {
				cadView.hasMouseFocus = true;
				updateCursorPosition(event);
				updateObjectsUnderMouse();
			}
			this.onMouseDown = function(event) {
				if(event.which == 1) {
					leftClick(event);
				}
				else if(event.which == 3) {
					rightClick(event);
				}
				/*
				this.onMouseMove(event);
				var activeObject;
				if(!event.ctrlKey) {
					cadView.selectedObjects.forEach(function(object) {
						if(object.deselect) object.deselect();
					});
					cadView.selectedObjects = [];
				}
				if(cadView.actionAreasUnderMouse.length > 0) {
					cadView.selectedObjects.push(cadView.actionAreasUnderMouse[0]);
					console.log("actionArea under mouse");
				}
				else if(cadView.objectsUnderMouse.length > 0) {
					//activeObject = cadView.objectsUnderMouse[0];
					cadView.selectedObjects.push(cadView.objectsUnderMouse[0]);
					if(cadView.objectsUnderMouse[0].select) cadView.objectsUnderMouse[0].select();
				}
				else {
					cadView.selectedObjects.forEach(function(object) {
						if(object.deselect) object.deselect();
					});
					cadView.selectedObjects = [];
				}

				console.log("selected objects: " + cadView.selectedObjects.length);

				cadView.redraw();

				//if(activeObject.onMouseDown) activeObject.onMouseDown(event);
				*/
			}
			this.onMouseUp = function(event) {
				this.onMouseMove(event);
				var activeObject;
				//if(cadView.actionAreasUnderMouse.length > 0) activeObject = cadView.actionAreasUnderMouse[0];
				if(cadView.objectsUnderMouse.length > 0) activeObject = cadView.objectsUnderMouse[0];
				else return;

				if(activeObject.onMouseUp) activeObject.onMouseUp(event);
			}
			this.onDoubleClick = function(event) {
				if(cadView.actionAreasUnderMouse.length > 0) {
					if(cadView.actionAreasUnderMouse[0].onDoubleClick) cadView.actionAreasUnderMouse[0].onDoubleClick(event);
				}
				else if(cadView.objectsUnderMouse.length > 0) {
					if(cadView.objectsUnderMouse[0].onDoubleClick) cadView.objectsUnderMouse[0].onDoubleClick(event);
				}
			}
		}

		this.cursorPosition = {x: 0, y: 0};
		this.objectsUnderMouse = [];
		this.actionAreasUnderMouse = [];

		var removeItemFromArray = function(item, array) {
			var i;
			if((i = array.indexOf(item)) == -1) return false;
			array.splice(i,1);
			return true;
		}

		this.removeActionArea = function(actionArea) {
			if(removeItemFromArray(actionArea, cadView.actionAreas)) {
				console.log("removed action area");
			}
			removeItemFromArray(actionArea, cadView.actionAreasUnderMouse);
			if(removeItemFromArray(actionArea, cadView.interactionLayer.objects)) {
				console.log("deleting action area; new number of action areas: " + cadView.actionAreas.length);
			}
		}
		this.removeObject = function(object) {
			if(!object) return;
			if(Array.isArray(object)) {
				object.forEach(function(shape) {
					cadView.removeObject(shape);
				});
			}
			else {
				if(object.actionAreas) {
					if(Array.isArray(object.actionAreas)) {
						object.actionAreas.forEach(function(actionArea) {
							cadView.removeActionArea(actionArea);
						});
					}
					else { 
						cadView.removeActionArea(actionArea);
					}
				}
				if(Array.isArray(object.shape)) {
					object.shape.forEach(function(shape) {
						cadView.removeObject(shape);
						delete shape;
					});
				}
				if(object.shape) {
					cadView.removeObject(object.shape);
					delete object.shape;
				}
				if(object.path) {
					delete object.path;
				}
				if(object.actionPoint) {
					if(Array.isArray(object.actionPoint)) {
						object.actionPoint.forEach(function(actionPoint) {
							cadView.removeActionArea(actionPoint);
						});
					}
					else {
						cadView.removeActionArea(object.actionPoint);
					}
				}
			}
			removeItemFromArray(object, cadView.objects);
		}

		var updateObjectsUnderMouse = function() {
			cadView.objectsUnderMouse = [];
			cadView.surfaceLayers.forEach(function(layer) {
				var invalidate = true;
				layer.objects.forEach(function(object) {
					if(cadView.hasMouseFocus && layer.isPointInShape(cadView.cursorPosition, object)) {
						object.isUnderMouse = true;
						cadView.objectsUnderMouse.push(object);
						if(object.onMouseOver) {
							object.onMouseOver();
							invalidate = true;
						}
					}
					else if(object.isUnderMouse) {
						object.isUnderMouse = false;
						if(object.onMouseOut) {
							object.onMouseOut();
							invalidate = true;
						}
					}
				});
				if(invalidate) layer.redrawLayer();
			});
			cadView.actionAreas.forEach(function(actionArea) {
				if(cadView.interactionLayer.isPointInShape(cadView.cursorPosition, actionArea)) {
					if(!actionArea.mouseOver) {
						actionArea.mouseOver = true;
						cadView.actionAreasUnderMouse.push(actionArea);
						if(actionArea.onMouseOver) actionArea.onMouseOver(event);
					}
				}
				else if(actionArea.mouseOver) {
					actionArea.mouseOver = false;
					removeItemFromArray(actionArea, cadView.actionAreasUnderMouse);
					if(actionArea.onMouseOut) actionArea.onMouseOut(event);
				}
			});
		}

		var onCursorMove = function(cursorPos) {
			cadView.updateGlobalPositions();
			updateObjectsUnderMouse();
		}

		var updateCursorPosition = function(event) {
			var newXPos, newYPos;
			if(cadView.snapping) {
				newXPos = Math.floor((event.offsetX+cadView.snapping.x/2)/cadView.snapping.x)*cadView.snapping.x;
				newYPos = Math.floor((event.offsetY+cadView.snapping.y/2)/cadView.snapping.y)*cadView.snapping.y;
			}
			else { 
				newXPos = event.offsetX;
				newYPos = event.offsetY;
			}

			if(cadView.cursorPosition.x != newXPos || cadView.cursorPosition.y != newYPos) {
				cadView.cursorPosition.x = newXPos;
				cadView.cursorPosition.y = newYPos;
				onCursorMove(cadView.cursorPosition);
			}
		}


		this.createActionArea = function(parameters) {
			var newActionArea = {};
			if(parameters) {
				for(var par in parameters) {
					newActionArea[par] = parameters[par];
				}
			}
			if(parameters.shape) {
				parameters.shape.call(newActionArea);
			}
			cadView.actionAreas.push(newActionArea);
			newActionArea.selected = false;
			newActionArea.select = function() {
				newActionArea.selected = true;
				newActionArea.strokeStyle = "green";
			}
			newActionArea.deselect = function() {
				newActionArea.selected = false;
				newActionArea.strokeStyle = undefined;
			}
			//newActionArea.strokeStyle = "green";
			cadView.interactionLayer.objects.push(newActionArea);
			return newActionArea;
		}

		this.updateGlobalPositions = function() {
			var updateGlobalPosition = function(object, currentGlobalPos, currentGlobalAngle) {
				if(Array.isArray(object)) {
					object.forEach(function(obj) {
						updateGlobalPosition(obj, currentGlobalPos, currentGlobalAngle);
					});
					return;
				}
				if(!object) return;
				if(!object.globalPos) object.globalPos = {x: currentGlobalPos.x, y: currentGlobalPos.y};
				if(object.pos) {
					var hyp = Math.sqrt(object.pos.x*object.pos.x+object.pos.y*object.pos.y);
					var ang = Math.atan2(object.pos.y, object.pos.x);
					object.globalPos.x = currentGlobalPos.x + hyp*Math.cos(currentGlobalAngle.value+ang);
					object.globalPos.y = currentGlobalPos.y + hyp*Math.sin(currentGlobalAngle.value+ang);
				}
				else {
					object.globalPos = {x: currentGlobalPos.x, y: currentGlobalPos.y};
				}
				if(object.angle) {
					if(typeof object.angle == "object") {
						object.globalAngle = {value: ((currentGlobalAngle.value + object.angle.value) % (Math.PI*2))};
					}
					else {
						object.globalAngle = {value: ((currentGlobalAngle.value + object.angle) % (Math.PI*2))};
					}
				}
				else object.globalAngle = {value: (currentGlobalAngle.value % (Math.PI*2))};
				if(object.shape) {
					if(!object.shape == undefined) return;
					if(Array.isArray(object.shape)) {
						object.shape.forEach(function(shape) {
							updateGlobalPosition(shape, object.globalPos, object.globalAngle);
						});
					}
					else {
						updateGlobalPosition(object.shape, object.globalPos, object.globalAngle);
					}
				}
			}
			cadView.objects.forEach(function(obj) {
				updateGlobalPosition(obj, {x: 0, y: 0}, {value:0});
			});
		}

		this.setEditorMode = function(newMode, parameters) {
			if(cadView.currentEditorMode == undefined) {
				cadView.currentEditorMode = newMode;
				return;
			}
			if(cadView.currentEditorMode.onLeaveMode) {
				cadView.currentEditorMode.onLeaveMode();
			}
			if(newMode.onEnterMode) newMode.onEnterMode(parameters);
			cadView.currentEditorMode = newMode;
		}

		this.setEditorMode(cadView.editorModes.idleMode);

		var cadViewDialogContainer = document.createElement("div");
		var cadViewDialog = new Dialog({
			content: cadViewDialogContainer,
			onShow: function() {
				cadView.suppressKeypress = true;
				cadView.suppressTab = false;
			},
			onHide: function() {
				cadView.suppressKeypress = false;
				cadView.suppressTab = true;
				while(cadViewDialogContainer.childElementCount) {
					cadViewDialogContainer.removeChild(cadViewDialogContainer.firstChild);
				}
			}
		});
		this.showDialog = function(dialogDomNode) {
			cadViewDialogContainer.appendChild(dialogDomNode);
			cadViewDialog.show();
		}
		this.closeDialog = function() {
			cadViewDialog.hide();
		}

		var mouseMoveTrigger = function(event) {
			updateCursorPosition(event);
			updateObjectsUnderMouse();
			if(cadView.currentEditorMode.onMouseMove) cadView.currentEditorMode.onMouseMove(event);

		}
		var mouseDownTrigger = function(event) {
			updateCursorPosition(event);
			updateObjectsUnderMouse();
			if(cadView.currentEditorMode.onMouseDown) cadView.currentEditorMode.onMouseDown(event);
		}
		var mouseUpTrigger = function(event) {
			updateCursorPosition(event);
			updateObjectsUnderMouse();
			if(cadView.currentEditorMode.onMouseUp) cadView.currentEditorMode.onMouseUp(event);
		}
		var dblclickTrigger = function(event) {
			if(cadView.currentEditorMode.onDoubleClick) cadView.currentEditorMode.onDoubleClick(event);
		}
		var mouseOutTrigger = function(event) {
			updateCursorPosition(event);
			updateObjectsUnderMouse();
			if(cadView.currentEditorMode.onMouseOut) cadView.currentEditorMode.onMouseOut(event);
		}
		var mouseOverTrigger = function(event) {
			updateCursorPosition(event);
			updateObjectsUnderMouse();
			if(cadView.currentEditorMode.onMouseOver) cadView.currentEditorMode.onMouseOver(event);
		}

		cadView.interactionLayer.addEventListener("mousemove", mouseMoveTrigger, false);
		cadView.interactionLayer.addEventListener("mousedown", mouseDownTrigger, false);
		cadView.interactionLayer.addEventListener("mouseup", mouseUpTrigger, false);
		cadView.interactionLayer.addEventListener("dblclick", dblclickTrigger, false);
		cadView.interactionLayer.addEventListener("mouseout", mouseOutTrigger, false);
		cadView.interactionLayer.addEventListener("mouseover", mouseOverTrigger, false);

	}
});
