
define(["widget/surfaceview", "widget/cadview/uicomponent", "widget/cadview/grid", "resources/resistor", "resources/capacitor"],
function(SurfaceView, uiComponent, Grid, Resistor, Capacitor) {
	return function(parentNode, args) {
		var surfaceView = new SurfaceView(parentNode, args);
		var context = surfaceView.context;
		var surface = surfaceView.surface;
		var mustache = surfaceView.mustache;
		var clearCanvas = surfaceView.clearCanvas;
		var drawShape = surfaceView.drawShape;
		var hasFocus = false;

		var snapping = args.snapping;

		var objects = [];
		var selectedObjects = [];
		var gui = [];

		var redraw = function() {
			clearCanvas();
			grid.rebuildGrid();
			drawShape(grid);
			objects.forEach(function(object) {
				drawShape(object);
			});
		}


		var onMouseLeave = function(event) {
			hasFocus = false;
			objects.forEach(function(object) {
				if(object.mouseIsOver) {
					object.mouseIsOver = false;
					if(object.activeActionArea) {
						if(object.activeActionArea.onMouseOut) {
							object.activeActionArea.onMouseOut();
						}
					}
					object.activeActionArea = undefined;
					if(object.onMouseOut) {
						object.onMouseOut();
					}
				}
			});
			redraw();
		}
		var onMouseEnter = function(event) {
			hasFocus = true;
			redraw();
		}

		var onMouseMove = function(event) {
			var invalidate = false;
			objects.forEach(function(object) {
				if(mustache.isPointInShape({x: event.offsetX, y: event.offsetY}, object)) {
					var actionArea = mustache.actionAreaAtPoint({x: event.offsetX, y: event.offsetY}, object);
					if(actionArea) {
						if(actionArea.onMouseOver) {
							actionArea.onMouseOver();
							invalidate = true;
						}
						object.activeActionArea = actionArea;
					}
					else { 
						if(object.activeActionArea) {
							if(object.activeActionArea.onMouseOut) {
								object.activeActionArea.onMouseOut();
								invalidate = true;
							}
						}
						object.activeActionArea = undefined;
					}
					object.mouseIsOver = true;
					if(object.onMouseOver) {
						object.onMouseOver();
						invalidate = true;
					}
				}
				else {
					if(object.mouseIsOver) {
						if(object.activeActionArea) {
							if(object.activeActionArea.onMouseOut) { 
								object.activeActionArea.onMouseOut();
								invalidate = true;
							}
						}
					}
					object.mouseIsOver = undefined;
					if(object.onMouseOut) {
						object.onMouseOut();
						invalidate = true;
					}
				}
			});
			if(invalidate) {
				redraw();
			}
		}

		var onMouseClick = function(event) {
			console.log("click!");
		}

		surface.addEventListener("mousemove", onMouseMove, false);
		surface.addEventListener("mouseout", onMouseLeave, false);
		surface.addEventListener("mouseover", onMouseEnter, false);
		surface.addEventListener("mouseclick", onMouseClick, false);
		surfaceView.addResizeEvent(redraw);

		var setView = function(center, scale) {
			grid.setView(center, scale);
		}

		var addEventListener = function(type, trigger, propagation) {

		}

		var grid = new Grid({surfaceView: surfaceView, meshWidth: 50, meshHeight: 50});

		return {
			"surface": surface,
			"objects": objects,
			"redraw": redraw,
			"addEventListener": addEventListener,
			"grid": grid
		}
	}
});
