

define(["widget/widgetbase", "mustache/mustache"],
function(WidgetBase, Mustache) {
	var Layer = function(name, order) {
		var newLayer = document.createElement("canvas");
		var mustache = new Mustache(newLayer);
		newLayer.setAttribute("style", "width: 100%; height: 100%; position: absolute; z-index: " + order + ";");
		newLayer.onmousedown = function() {
			return false;
		}
		newLayer.oncontextmenu = function() {
			return false;
		}
		newLayer.objects = [];
		newLayer.name = name;
		newLayer.removeObject = function(object) {
			var i;
			if((i = newLayer.objects.indexOf(object)) == -1) return;
			newLayer.objects.splice(i,1);
		}
		newLayer.draw = function(object) {
			mustache.drawShape(object);
		}
		newLayer.clear = function() {
			mustache.clearCanvas();
		}
		newLayer.redrawLayer = function() {
			mustache.clearCanvas();
			newLayer.objects.forEach(function(object) {
				newLayer.draw(object);
			});
		}
		newLayer.isPointInShape = mustache.isPointInShape;
		return newLayer;
	}
	return function() {
		var surfaceView = this;
		if(!this.widgetBaseMixin) WidgetBase.call(this);
		this.surfaceViewMixin = true;
		this.surfaceLayers = [];
		this.objects = [];
		this.onSurfaceLayerCreated = [];
		this.createLayer = function(name, order) {
			var newLayer = new Layer(name, order);
			newLayer.width = this.widgetWidth;
			newLayer.height = this.widgetHeight;
			this.content.appendChild(newLayer);
			this.surfaceLayers.push(newLayer);
			surfaceView.onResize.push(function(event) {
				newLayer.redrawLayer();
			});
			this.onSurfaceLayerCreated.forEach(function(func) {
				func.call(surfaceView, newLayer);
			});
			return newLayer;
		}
		this.redraw = function() {
			surfaceView.surfaceLayers.forEach(function(layer) {
				layer.redrawLayer();
			});
		}
		this.onResize.push(function(event) {
			surfaceView.surfaceLayers.forEach(function(layer) {
				layer.width = event.w;
				layer.height = event.h;
			});
		});
	}
});
